'use strict'
const path = require('path')

const px2rem = require('postcss-px2rem')
// 配置基本大小
const postcss = px2rem({
  // 基准大小 baseSize，需要和rem.js中相同
  remUnit: 80
})

function resolve(dir) {
  return path.join(__dirname, dir)
}
const name = '重卡换电智能站控系统'
module.exports = {
  publicPath: "/",
  outputDir: 'dist',
  assetsDir: 'static',
  lintOnSave: false,
  css: {
    loaderOptions: {
      postcss: {
        plugins: [
          postcss
        ]
      }
    }
  },
  productionSourceMap: false,
  devServer: {
    host: 'localhost',
    port: 808,
    open: true,
    overlay: {
      warning: false,
      error: true,
    },
    proxy: {
      '/api': {
        // target: `http://192.168.3.177:8080`,
        target: `http://zk.li-ai.com.cn:8080`,
        secure: false,
        ws: true,
        changeOrigin: true,
      },
      '/ws': {
        // target: `ws://192.168.3.177:8080/ws`,
        target: `ws://zk.li-ai.com.cn:8080/ws`,
        secure: false,
        ws: true,
        changeOrigin: true,
      },
      '/video':{
        // target: `ws://192.168.3.177:8082/`,
        target: `ws://192.168.3.177:8082/ws`,
        secure: false,
        ws: true,
        changeOrigin: true,
      }
    },
  },
  
  configureWebpack: {
    name: name,
    resolve: {
      alias: {
        '@': resolve('src')
      }
    }
  },
  chainWebpack(config) {
    config.plugin('preload').tap(() => [
      {
        rel: 'preload',
        fileBlacklist: [/\.map$/, /hot-update\.js$/, /runtime\..*\.js$/],
        include: 'initial'
      }
    ])

    config.plugins.delete('prefetch')
    config.module
      .rule('svg')
      .exclude.add(resolve('src/icons'))
      .end()
    config.module
      .rule('icons')
      .test(/\.svg$/)
      .include.add(resolve('src/icons'))
      .end()
      .use('svg-sprite-loader')
      .loader('svg-sprite-loader')
      .options({
        symbolId: 'icon-[name]'
      })
      .end()

    config
      .when(process.env.NODE_ENV !== 'development',
        config => {
          config
            .plugin('ScriptExtHtmlWebpackPlugin')
            .after('html')
            .use('script-ext-html-webpack-plugin', [{
              inline: /runtime\..*\.js$/
            }])
            .end()
          config
            .optimization.splitChunks({
              chunks: 'all',
              cacheGroups: {
                libs: {
                  name: 'chunk-libs',
                  test: /[\\/]node_modules[\\/]/,
                  priority: 10,
                  chunks: 'initial' 
                },
                elementUI: {
                  name: 'chunk-elementUI', 
                  priority: 20, 
                  test: /[\\/]node_modules[\\/]_?element-ui(.*)/ 
                },
                commons: {
                  name: 'chunk-commons',
                  test: resolve('src/components'), 
                  minChunks: 3, 
                  priority: 5,
                  reuseExistingChunk: true
                }
              }
            })
          config.optimization.runtimeChunk('single')
        }
      )
  }
}